package com.alterdekim.flash.decompiler.tag;

import com.alterdekim.flash.decompiler.action.ActionConstantPool;
import com.alterdekim.flash.decompiler.action.ActionPush;
import com.alterdekim.flash.decompiler.action.ByteCodeAction;
import com.alterdekim.flash.decompiler.compiler.CompileObject;
import com.alterdekim.flash.decompiler.item.ActionPushType;
import com.alterdekim.flash.decompiler.util.ActionField;
import com.alterdekim.flash.decompiler.util.ByteUtil;
import lombok.*;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@NoArgsConstructor
@ToString
@Getter
@Setter
@AllArgsConstructor
public class DoAction extends ShockwaveTag implements CompileObject {

    private List<String> actionPool;
    private List<ByteCodeAction> actions;

    @Override
    public TagType getType() {
        return TagType.DoAction;
    }

    @Override
    public List<Byte> compile() {
        List<Byte> bytes = new ArrayList<>();
        this.actions.forEach(action -> {
            bytes.add(action.getType().id);
            List<Byte> data = new ArrayList<>();
            switch(action.getType()) {
                case Push:
                    ActionPush push = (ActionPush) action;
                    push.getItems().forEach(item -> {
                        data.add((byte) item.getType().type);
                        switch (item.getType()) {
                            case STRING:
                                data.addAll(fromString((String) item.getValue()));
                                break;
                            case INTEGER:
                                data.addAll(ByteUtil.intToBytes32((Integer) item.getValue()));
                                break;
                            case POOL_INDEX: // todo: make distiction between small POOL_INDEX and large POOL_INDEX
                                Integer index = (Integer) item.getValue();
                                if( index >= 256 ) {
                                    data.remove(data.size()-1);
                                    data.add((byte) ActionPushType.POOL_INDEX_BIG.type);
                                }
                                data.addAll(index < 256 ? ByteUtil.intToBytes8(index) : ByteUtil.intToBytes(index));
                                break;
                            case DOUBLE:
                                data.addAll(ByteUtil.doubleToBytes((Double) item.getValue()));
                                break;
                            case BOOLEAN:
                                data.add((byte) ((Boolean) item.getValue() ? 0x01 : 0x00));
                                break;
                        }
                    });
                    bytes.addAll(ByteUtil.intToBytes(data.size()));
                    bytes.addAll(data);
                    break;
                case ConstantPool:
                    ActionConstantPool pool = (ActionConstantPool) action;
                    data.addAll(ByteUtil.intToBytes(pool.getPool().size()));
                    pool.getPool().forEach(s -> data.addAll(fromString(s)));
                    bytes.addAll(ByteUtil.intToBytes(data.size()));
                    bytes.addAll(data);
                    break;
            }
        });
        bytes.add((byte) 0x0);
        int val = getType().getId() << 6;
        val |= 63; // 265955
        return Stream.of( ByteUtil.intToBytes(val), ByteUtil.intToBytes32(bytes.size()), bytes ).flatMap(Collection::stream).collect(Collectors.toList());
    }

    private List<Byte> fromString(String s) {
        List<Byte> l = new ArrayList<>();
        for( byte c : s.getBytes() ) {
            l.add(c);
        }
        l.add((byte) 0x0);
        return l;
    }
}
